/**
 *
 */
package pl.koziolekweb.programmers.zip;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * Abstrakcyjna klasa do obsługi archiwizacji za pomocą ZIP. Klasy rozszerzające
 * muszą nadpisać metodę {@link #makeStreams(File)}.
 * 
 * @author koziolek
 * 
 */
public abstract class AchiverAbstractZIPImpl implements ZipArchiver {

	protected static final int BUFFER = 2048;

	protected FileOutputStream dest;

	protected ZipOutputStream out;

	protected BufferedInputStream entryStream;

	protected String basePath;

	protected LEVEL compresionLevel;

	protected String message;

	/*
	 * (non-Javadoc)
	 * 
	 * @see pl.koziolekweb.programmers.zip.Archiver#archive(File, File)
	 */
	@Override
	public void archive(File source, File target) throws IOException {
		if (!target.exists()) {
			target.createNewFile();
		}
		basePath = source.getAbsolutePath();

		makeStreams(target);
		setMethod();
		if (message != null) {
			out.setComment(message);
		}

		if (source.isDirectory()) {
			archiveDir(source);
		} else {
			archiveFile(source);
		}
		out.close();
	}

	/**
	 * Konfiguruje strumienie.
	 * 
	 * @param target
	 *            plik archiwum.
	 * @throws FileNotFoundException
	 *             jeżeli nie istnieje plik wynikowy.
	 */
	protected abstract void makeStreams(File target)
			throws FileNotFoundException;

	/**
	 * Ustawia metodę w zależności od poziomu kompresji.
	 */
	private void setMethod() {
		if (this.compresionLevel != LEVEL._0) {
			out.setMethod(ZipOutputStream.DEFLATED);
		} else {
			out.setMethod(ZipOutputStream.STORED);
		}
	}

	/**
	 * Przetwarza rekurencyjnie folder jeżeli próbujemy dodać go do archiwum.
	 * Jeżeli przekażemy do metody zwykły plik to zostanie on zignorowany.
	 * 
	 * @param file
	 *            folder.
	 * @throws IOException
	 */
	private void archiveDir(File file) throws IOException {
		if (file.isFile()) {
			System.out.println("Ignoruję zwykły plik: "
					+ file.getAbsolutePath());
		}
		for (File f : file.listFiles()) {
			if (f.isDirectory()) {
				archiveDir(f);
			} else {
				archiveFile(f);
			}
		}
	}

	/**
	 * Dodaje wskazany plik do archiwum.
	 * 
	 * @param file
	 *            plik do dodania.
	 * @throws IOException
	 */
	private void archiveFile(File file) throws IOException {
		String targetFilePath = file.getAbsolutePath().replace(basePath, "");
		byte data[] = new byte[BUFFER];

		FileInputStream fi = new FileInputStream(file);
		entryStream = new BufferedInputStream(fi, BUFFER);
		ZipEntry entry = new ZipEntry(targetFilePath);
		out.putNextEntry(entry);
		int count;
		while ((count = entryStream.read(data, 0, BUFFER)) != -1) {
			out.write(data, 0, count);
		}
		entryStream.close();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * pl.koziolekweb.programmers.zip.ZipArchiver#setMessage(java.lang.String)
	 */
	@Override
	public void setComment(String message) {
		this.message = message;
	}
}